package com.agfa.wildfly.interceptor.mdb.cdi;

import java.lang.annotation.Annotation;
import java.util.HashSet;

import javax.ejb.MessageDriven;
import javax.ejb.Stateless;
import javax.enterprise.event.Observes;
import javax.enterprise.inject.spi.AnnotatedType;
import javax.enterprise.inject.spi.Extension;
import javax.enterprise.inject.spi.ProcessAnnotatedType;
import javax.enterprise.inject.spi.WithAnnotations;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class LoggerCDIExtension implements Extension {
    private final static Logger LOGGER = LoggerFactory.getLogger(LoggerCDIExtension.class);
    
    /**
     * Decorates MDBs by adding a Logged annotation on them so that their onMessage will be intercepted
     * @param mdbType a MDB process type
     */
    <T> void processMDBs(@Observes @WithAnnotations(MessageDriven.class) ProcessAnnotatedType<T> mdbType) {
        decorate(mdbType);
    }

    /**
     * Decorates Stateless EJB by adding a Logged annotation
     * @param statelessType a Stateless type to process
     */
    <T> void processStateless(@Observes @WithAnnotations(Stateless.class) ProcessAnnotatedType<T> statelessType) {
        decorate(statelessType);
    }

    
//    <T> void processBean(@Observes ProcessBean<T> bean) {
//        LOGGER.error("CDI process bean: {}", bean.getAnnotated().getBaseType().toString());
//    }
    
    private <T> void decorate(ProcessAnnotatedType<T> mdbType) {
        Annotation loggedAnnotation = new Annotation() {
            @Override
            public Class<? extends Annotation> annotationType() {
                return Logged.class;
            }
        };
        
        AnnotatedType<T> annotatedType = mdbType.getAnnotatedType();
        AnnotatedTypeWrapper<T> wrapper = new AnnotatedTypeWrapper<>(annotatedType, new HashSet<>());
        wrapper.addAnnotation(loggedAnnotation);
        
        LOGGER.debug("adding Logged annotation on class {}", mdbType.getAnnotatedType().getJavaClass());
        mdbType.setAnnotatedType(wrapper);
    }

}
